import unittest
from acme.domain.shift import Shift
from acme.domain.working_hours import WorkingHours


class TestWorkingHours(unittest.TestCase):

    def setUp(self):
        self.working_hours = WorkingHours('01:00-03:00')
        self.shift = Shift(name='first_shift', hours='00:01-09:00')

    def test_start_time_is_between(self):
        is_between = self.working_hours.start_time_is_between(self.shift)

        self.assertTrue(is_between)

    def test_check_hours_exception(self):
        with self.assertRaises(SystemExit) as cm:
            wrong_working_hours = WorkingHours('00:00-03:00')

        self.assertEqual(cm.exception.code, 1)

        with self.assertRaises(SystemExit) as cm:
            wrong_working_hours = WorkingHours('25:00-40:00')

        self.assertEqual(cm.exception.code, 1)

        with self.assertRaises(SystemExit) as cm:
            wrong_working_hours = WorkingHours('04:00-03:00')

        self.assertEqual(cm.exception.code, 1)



import sys
import logging
from typing import List
from acme.data import constants
from acme.expections import MalformedFileError
from acme.data.constants import WEEKEND, WORKWEEK


def get_day_abbrev(day):

    day_abbrev = day[:2]

    if day_abbrev not in WORKWEEK:
        if day_abbrev not in WEEKEND:
            raise MalformedFileError('Error obtaining day name')

    return day_abbrev


def get_employee_name(text_line):

    employee_name = text_line.split('=')[0]

    return employee_name


def get_day_times(day_worked):

    times_worked = day_worked[2:]

    return times_worked


def get_hours_minutes(hours):

    try:
        start_time, end_time = hours.split('-')
    except ValueError as e:
        raise MalformedFileError('Error obtaining hours and minutes times')

    return start_time, end_time


def get_week_worked(text_line):

    try:
        week_worked = text_line.split('=')[1]
    except IndexError:
        raise MalformedFileError('Error obtaining week worked')

    return week_worked


def to_decimal_hours(time: str) -> float:
    """Pass hours from string format HH:MM to hours decimal"""

    try:
        h, m = time.split(':')
        hours = int(h) + int(m) / 60
    except ValueError:
        print(time)
        raise MalformedFileError('Error parsing hours')

    return hours


def hours_to_string(hours: float) -> str:

    h = int(hours)
    m = int((hours - h) * 60)

    h = str(h)
    m = str(m)

    if h == '0':
        h += '0'

    if m == '0':
        m += '0'

    return f'{h}:{m}'



import unittest
from acme.expections import MalformedFileError, WrongTimeRangeError


class TestExceptions(unittest.TestCase):

    def test_malformed_file_error(self):
        with self.assertRaises(SystemExit) as cm:
            raise MalformedFileError('Error obtaining name')

        self.assertEqual(cm.exception.code, 1)

    def test_wrong_time_range_error(self):
        with self.assertRaises(SystemExit) as cm:
            raise WrongTimeRangeError(4.0, 0.3)

        self.assertEqual(cm.exception.code, 1)



from acme.data.constants import WEEKEND, ONE_MINUTE
from acme.domain.working_hours import WorkingHours
from acme.data.shiftwork import shiftwork
from acme.data.rates import rates


class Day:

    def __init__(self, day_abbrev: str):
        self.day_abbrev = day_abbrev
        self._week = self._get_week()

    def _get_week(self) -> str:
        week = 'workweek'

        if self.day_abbrev in WEEKEND:
            week = 'weekend'

        return week

    def calculate_day_cost(self, hours: str) -> float:

        working_hours = WorkingHours(hours)
        cost: float = 0
        worked_hours: float = 0
        plus_one = False

        for shift in shiftwork:

            rate = rates[shift.name][self._week]

            if working_hours.start_time_is_between(shift):

                if working_hours.end_time <= shift.end_time:
                    worked_hours = working_hours.end_time - working_hours.start_time
                    cost += worked_hours * rate
                    if plus_one:
                        cost += ONE_MINUTE * rate
                        plus_one = False

                else:
                    worked_hours = shift.end_time - working_hours.start_time
                    cost += worked_hours * rate
                    working_hours.start_time = shift.end_time + ONE_MINUTE
                    plus_one = True

        return cost



import io
import os
import re
import unittest
from acme.commands import processing
import contextlib as cl


class TestCommands(unittest.TestCase):

    def setUp(self):
        # Check if SO is Windows
        if os.name == 'nt':
            self.salary_file_path = 'data\\salaries.txt'
            self.employees_file_path = 'acme\\data\\employees.txt'
        else:
            self.salary_file_path = 'data/salaries.txt'
            self.employees_file_path = 'acme/data/employees.txt'

    def test_processing_wrong_arguments(self):

        tests = [
            ['-casdfdsa'],
            ['data.bin'],
            ['-helpasd'],
        ]

        for test in tests:

            out = io.StringIO()
            with cl.redirect_stdout(out):
                processing(test)
            data = out.getvalue()

            err = 'Invalid argument.\nFor more information run: acme --help\n'
            self.assertEqual(data, err)

    def test_acme_command(self):
        test = [self.employees_file_path]

        # Get the console output
        out = io.StringIO()
        with cl.redirect_stdout(out):
            processing(test)
        content = out.getvalue()

        # Read sample salaries
        file_path = os.path.dirname(os.path.realpath(__file__))
        path: str = os.path.join(file_path, self.salary_file_path)
        with open(path) as file:
            expected_content = file.read()

        self.assertEqual(content, expected_content)

    def test_acme_instructions(self):
        test = ['--help']

        # Get the console output
        out = io.StringIO()
        with cl.redirect_stdout(out):
            processing(test)
        instructions = re.sub(r'[\n\t\s]', '', out.getvalue())

        # Read instructions
        with open('tests/data/instructions.txt') as f:
            expected_instructions = re.sub(r'[\n\t\s]', '', f.read())

        # Allow long text
        self.maxDiff = None

        self.assertEqual(instructions, expected_instructions)

    def test_acme_demo_command(self):
        test = ['--demo']

        # Get the console output
        out = io.StringIO()
        with cl.redirect_stdout(out):
            processing(test)
        demo_content = out.getvalue()

        # Read sample employees data
        file_path = os.path.dirname(os.path.realpath(__file__))
        path: str = os.path.join(file_path, self.salary_file_path)
        with open(path) as file:
            expected_content = file.read()

        self.assertEqual(demo_content, expected_content)



from acme import utils
from acme.domain.day import Day
from acme.expections import MalformedFileError


class Employee:
    salary = 0

    def __init__(self, name):
        self.name = name

    def calculate_salary(self, week_worked: str) -> float:
        salary: float = 0.0
        datetimes = week_worked.split(',')
        if '' in datetimes:
            raise MalformedFileError('Error obtaining datetimes')
        else:
            for datetime in datetimes:
                day_times = utils.get_day_times(datetime)
                day_abbrev = utils.get_day_abbrev(datetime)
                day = Day(day_abbrev)
                salary += day.calculate_day_cost(day_times)

        self.salary = salary

        return salary



import os
import unittest
from acme.files import get_employees_from_file, get_file_path, read_file_lines


class TestFiles(unittest.TestCase):

    def setUp(self):
        # Check if OS is Windows
        if os.name == 'nt':
            self.employees_file_path = 'acme\\data\\employees.txt'
            self.empty_file_path = 'acme\\data\\empty_file.txt'
        else:
            self.employees_file_path = 'acme/data/employees.txt'
            self.empty_file_path = 'acme/data/empty_file.txt'

    def test_get_employees_from_file(self):
        employees: str = get_employees_from_file(self.employees_file_path)
        employees_names = [employee.name for employee in employees]

        for employee in employees:
            self.assertTrue('RENE' in employees_names)
            self.assertTrue('ASTRID' in employees_names)
            self.assertTrue('WLADYMIR' in employees_names)

    def test_find_file(self):
        current_dir = os.getcwd()
        filepath: str = get_file_path(self.employees_file_path)
        expected_file_path: str = os.path.join(
            current_dir, self.employees_file_path)

        self.assertEqual(filepath, expected_file_path)

    def test_read_file_lines(self):
        path: str = get_file_path(self.employees_file_path)
        file_lines: str = read_file_lines(path)
        with open(path) as file:
            expected_content = file.readlines()

        self.assertEqual(file_lines, expected_content)

    def test_read_file_lines_exception(self):
        path: str = get_file_path(self.empty_file_path)

        with self.assertRaises(SystemExit) as cm:
            read_file_lines(path)

        self.assertEqual(cm.exception.code, 1)



import os
from acme.data.constants import FILE_NOT_FOUND, EMPTY_FILE
from acme.utils import get_employee_name, get_week_worked
from acme.domain.employee import Employee
from acme.expections import EmptyFileError


def get_employees_from_file(file):
    path: str = get_file_path(file)
    employees = []
    text_lines = read_file_lines(path)
    for text_line in text_lines:
        employee_name: str = get_employee_name(text_line)
        week_worked: str = get_week_worked(text_line)
        employee = Employee(employee_name)
        employee.calculate_salary(week_worked)
        employees.append(employee)

    return employees


def get_file_path(relative_path: str) -> str:
    current_dir = os.getcwd()
    new_path: str = os.path.join(current_dir, relative_path)

    return new_path


def read_file_lines(path: str) -> str:

    lines = []

    try:
        with open(path) as file:
            lines = file.readlines()

    except FileNotFoundError:
        print(FILE_NOT_FOUND)

    if not lines:
        raise EmptyFileError()

    return lines


def write_employees_data():
    dirname = os.path.dirname(os.path.realpath(__file__))
    employess_file_path = os.path.join(dirname, "data/employees.txt")
    with open(employess_file_path, "r") as base_file:
        content = base_file.read()
        with open("employees.txt", "w+") as file:
            file.write(content)


import unittest
from acme import utils


class TestUtils(unittest.TestCase):

    def test_get_hours_and_minutes(self):

        start_time, end_time = utils.get_hours_minutes('10:00-12:00')

    def test_get_hours_and_minutes_exception(self):
        with self.assertRaises(SystemExit) as cm:
            start_time, end_time = utils.get_hours_minutes('10:0012:00')

        self.assertEqual(cm.exception.code, 1)

    def test_get_day_times(self):

        expected_hours = '05:00-07:00'
        hours = utils.get_day_times('MO05:00-07:00')

        self.assertEqual(hours, expected_hours)

    def test_get_day_abbrev(self):

        expected_day_abbrev = 'MO'
        day = utils.get_day_abbrev('MO05:00-07:00')

        self.assertEqual(day, expected_day_abbrev)

    def test_get_day_abbrev_exception(self):

        with self.assertRaises(SystemExit) as cm:
            day = utils.get_day_abbrev('05:00-07:00')

        self.assertEqual(cm.exception.code, 1)

    def test_get_employee_name(self):

        expected_name = 'ASTRID'
        name = utils.get_employee_name(
            'ASTRID=MO10:00-12:00,TH12:00-14:00,SU20:00-21:00')

        self.assertEqual(name, expected_name)

    def test_get_week_worked(self):

        expected_week = 'MO10:00-12:00,TH12:00-14:00,SU20:00-21:00'
        week_worked = utils.get_week_worked(
            'ASTRID=MO10:00-12:00,TH12:00-14:00,SU20:00-21:00')

        self.assertEqual(week_worked, expected_week)

    def test_get_week_worked_exception(self):
        with self.assertRaises(SystemExit) as cm:
            start_time, end_time = utils.get_week_worked('ASTRID')

        self.assertEqual(cm.exception.code, 1)

    def test_to_decimal_hours(self):

        expected_decimal = 10.0
        decimal_hours = utils.to_decimal_hours('10:00')

        self.assertEqual(decimal_hours, expected_decimal)

    def test_to_decimal_hours_exception(self):

        with self.assertRaises(SystemExit) as cm:
            start_time, end_time = utils.to_decimal_hours('00')

        self.assertEqual(cm.exception.code, 1)

    def test_hours_to_string(self):

        expected_hours = '20:00'
        hours = utils.hours_to_string(20.0)

        self.assertEqual(hours, expected_hours)

        expected_hours = '00:00'
        hours = utils.hours_to_string(0)

        self.assertEqual(hours, expected_hours)



ONE_MINUTE = 1 / 60

SUCCESS = 1
FAIL = 0

FILE_NOT_FOUND = 'File not found.\nBe sure the file exists.'
MALFORMED_FILE = 'File is malformed, please check the file'
WRONG_TIME_RANGE = 'Invalid range of hours'
EMPTY_FILE = 'File is empty'

WORKWEEK = ['MO', 'TU', 'WE', 'TH', 'FR', 'SA', 'SU']
WEEKEND = ['SA', 'SU']

MIN_HOUR = 0
MAX_HOUR = 24



import sys
import logging
from acme.data.constants import MALFORMED_FILE, WRONG_TIME_RANGE, EMPTY_FILE
from acme import utils


class MalformedFileError(Exception):

    def __init__(self, message):
        self.message = message
        logging.error(self.message)
        logging.error(MALFORMED_FILE)
        exit(1)


class WrongTimeRangeError(Exception):

    def __init__(self, start_time, end_time):
        first_time = utils.hours_to_string(start_time)
        second_time = utils.hours_to_string(end_time)
        logging.error(f'{WRONG_TIME_RANGE}: {first_time}-{second_time}')
        sys.exit(1)


class EmptyFileError(Exception):

    def __init__(self):
        logging.error(EMPTY_FILE)
        exit(1)



from acme.data.constants import SUCCESS, FAIL
from acme.files import get_employees_from_file, write_employees_data


def processing(argv):
    """Processing the command and check argument"""

    options = ['-d', '--demo', '-h', '--help']
    is_option = argv[0] in options or argv[0].endswith('.txt')

    completed = FAIL

    if is_option:
        completed = check(argv)

    err = {
        FAIL:  'Invalid argument.\nFor more information run: acme --help'
    }

    if completed != SUCCESS:
        print(err.get(completed))


def check(argv):
    """Check if some command is completed"""

    completed = 0

    if len(argv) == 1:
        if argv[0] == '-h' or argv[0] == '--help':
            instructions()
            completed = SUCCESS
        elif argv[0] == '-d' or argv[0] == '--demo':
            completed = demo()
        else:
            completed = run(argv[0])

    return completed


def run(file):
    completed = 0
    employees = get_employees_from_file(file)
    if employees:
        for employee in employees:
            print('The amount to pay {} is: {:.2f} USD'.format(
                employee.name, employee.salary)
            )
        completed = 1

    return completed


def demo():
    completed = 0
    write_employees_data()
    completed = run('employees.txt')

    return completed


def instructions():
    """Print the instructions"""

    info = r"""
      ______    ______   __       __  ________
     /      \  /      \ |  \     /  \|        \
    |  $$$$$$\|  $$$$$$\| $$\   /  $$| $$$$$$$$
    | $$__| $$| $$   \$$| $$$\ /  $$$| $$__
    | $$    $$| $$      | $$$$\  $$$$| $$  \
    | $$$$$$$$| $$   __ | $$\$$ $$ $$| $$$$$
    | $$  | $$| $$__/  \| $$ \$$$| $$| $$_____
    | $$  | $$ \$$    $$| $$  \$ | $$| $$     \
     \$$   \$$  \$$$$$$  \$$      \$$ \$$$$$$$$"""
    usage = """
    Usage:
        acme <file>
        
        Ensure the file is a text (.txt) file.
        In addition, that the file is in the same
        directory where the command is executed."""

    options = """
    Options:
        -d, --demo  : Create a demo file (employees.txt) 
                      and execute acme employees.txt.
        -h, --help  : Print the instructions."""

    examples = """
    Examples:
        acme data.txt
        acme --demo
        acme -h"""

    help = (
        f'{info}\n'
        f'{usage}\n'
        f'{options}\n'
        f'{examples}'
    )

    print(help)



import unittest
from acme.domain.day import Day


class TestDay(unittest.TestCase):

    def setUp(self):
        self.day = Day('MO')

    def test_calculate_day_cost(self):
        day_cost: float = self.day.calculate_day_cost('10:00-12:00')
        self.assertEqual(day_cost, 30.0)
        day_cost: float = self.day.calculate_day_cost('8:00-17:00')
        self.assertEqual(day_cost, 145.0)



from acme.utils import to_decimal_hours, get_hours_minutes
from acme.data.constants import MAX_HOUR, MIN_HOUR
from acme.expections import WrongTimeRangeError


class WorkingHours:

    def __init__(self, hours: str):
        self.start_time, self.end_time = get_hours_minutes(hours)
        self._to_hours()

    def _to_hours(self):
        self.start_time = to_decimal_hours(self.start_time)
        self.end_time = to_decimal_hours(self.end_time)
        self._check_hours()

    def _check_hours(self):
        if self.start_time == 0:
            raise WrongTimeRangeError(self.start_time, self.end_time)

        if self.end_time == 0:
            self.end_time = 24.0

        if self.start_time > MAX_HOUR or self.end_time > MAX_HOUR:
            raise WrongTimeRangeError(self.start_time, self.end_time)

        if self.start_time > self.end_time:
            if self.end_time > MIN_HOUR:
                raise WrongTimeRangeError(self.start_time, self.end_time)

    def start_time_is_between(self, shift):
        ans = False

        if self.start_time >= shift.start_time:
            if self.start_time <= shift.end_time:
                ans = True

        return ans



rates = {
    'first_shift': {
        'workweek': 25,
        'weekend': 30
    },
    'second_shift': {
        'workweek': 15,
        'weekend': 20
    },
    'third_shift': {
        'workweek': 20,
        'weekend': 25
    }
}



import unittest
from acme.domain.employee import Employee


class TestEmploye(unittest.TestCase):

    def setUp(self):
        self.employee = Employee('ASTRID')

    def test_calculate_salary_amount(self):
        salary: float = self.employee.calculate_salary(
            'MO10:00-12:00,TU10:00-12:00,TH01:00-03:00,SA14:00-18:00,SU20:00-21:00')
        self.assertEqual(salary, 215.0)
        salary: float = self.employee.calculate_salary(
            'MO10:00-12:00,TH12:00-14:00,SU20:00-21:00')
        self.assertEqual(salary, 85.0)
        salary: float = self.employee.calculate_salary(
            'MO5:00-11:00,TH8:00-17:00,SU01:00-05:00')
        self.assertEqual(salary, 395.0)

    def test_calculate_salary_amount_exception(self):
        with self.assertRaises(SystemExit) as cm:
            self.employee.calculate_salary('MO10:00-12:00,,')

        self.assertEqual(cm.exception.code, 1)



from acme.domain.shift import Shift

# Shift start at midnight and ends in the morning
first_shift = Shift(name='first_shift', hours='00:01-09:00')

# Working during the day
second_shift = Shift(name='second_shift', hours='09:01-18:00')

# Runs from afternoon to midnight
third_shift = Shift(name='third_shift', hours='18:01-00:00')

# Shifts of the workday
shiftwork = [first_shift, second_shift, third_shift]



from setuptools import setup, find_packages

from acme import __version__

with open('README.md') as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()

setup(
    name="acme-exercise",
    version=__version__,
    description="Console app to calculate the total that the company has to pay an employee.",
    license="MIT",
    author="Israel Teneda",
    author_email="israteneda@gmail.com",
    url="https://github.com/israteneda/acme",
    packages=find_packages(),
    package_data={'acme': ['data/employees.txt']},
    entry_points={
        "console_scripts": [
            "acme=acme.__main__:main"
        ]
    },
    test_suite="tests",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License"
    ],
    python_requires=">=3.7",
    project_urls={
        "Homepage": "https://github.com/israteneda/acme",
        "Source Code": "https://github.com/israteneda/acme",
    },
)



from acme.commands import processing, instructions
import sys


def main():
    argv = sys.argv[1:]
    if len(argv) >= 1:
        processing(argv)
    else:
        instructions()


if __name__ == '__main__':
    main()
